unit GTNVolaSMS;

interface

uses
  SysUtils, Classes, IdGlobal, IdURI, IdBaseComponent, IdComponent, IdTCPConnection,
  IdTCPClient, IdHTTP, IdMultiPartFormData, DateUtils;

  type
    TSms = Class
     SenderPhone : String;
     DestPhone : String;
     Msg : String;
     DateTime : TDateTime;
  end;

  type TSmsList = array of TSms;

  type
    TSmsResponse = Class
     Code : String;
     SentNumErr : integer;
     SentListErr : String;
     SmsIncoming: TSmsList;
     ErrMsg : String;
  end;

type
  TVolaSMS = class(TIdCustomHTTP)
  private
    function Get_VolaAppVer: String;
  private
    { Private declarations }
    sVolaUser: String;
    sVolaPwd: String;
    sVolaVAPCODE: String;
    sVolaTestMode: boolean;
    property VolaAppVer: String read Get_VolaAppVer;
    function Get_VolaPwd: String;
    function Get_VolaUser: String;
    procedure Set_VolaPwd(Const Value: String);
    procedure Set_VolaUser(Const Value: String);
    function SetAppVersion: TSmsResponse;
    function Get_Host: String;
    function Get_VolaVAPCODE: String;
    procedure Set_VolaVAPCODE(const Value: String);
    function Get_VolaTestMode: boolean;
    procedure Set_VolaTestMode(const Value: boolean);
    function ParseResult(Value: String): TSmsResponse;
    function Implode(lst:  TStringList; sep: String= ';'): String;
    function Explode(ch: String; sep: String= ';'): TStringList;
    function AckIncomingSms: TSmsResponse;
  protected
    { Protected declarations }

  public
    { Public declarations }
    constructor create(AOwner: TComponent); override;
    function SendSms(Sms: TSms): TSmsResponse;
    function GetCredit: Integer;
    function GetIncomingSms: TSmsResponse;

  published
    { Published declarations }
    property Host: String read Get_Host;
    property VolaUser: String read Get_VolaUser write Set_VolaUser;
    property VolaPwd: String read Get_VolaPwd write Set_VolaPwd;
    property VolaVAPCODE: String read Get_VolaVAPCODE write Set_VolaVAPCODE;
    property VolaTestMode: boolean read Get_VolaTestMode write Set_VolaTestMode default false;

  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('GTN Controls', [TVolaSMS]);
end;

{ TVolaSMS }

constructor TVolaSMS.create(AOwner: TComponent);
begin
  inherited;
end;


function TVolaSMS.Get_Host: String;
begin
  Result:= 'www.vola.it';
end;


function TVolaSMS.Get_VolaPwd: String;
begin
  Result:= sVolaPwd;
end;

function TVolaSMS.Get_VolaUser: String;
begin
  Result:= sVolaUser;
end;

procedure TVolaSMS.Set_VolaPwd(const Value: String);
begin
   sVolaPwd := Value;
end;

procedure TVolaSMS.Set_VolaUser(const Value: String);
begin
   sVolaUser := Value;
end;


function TVolaSMS.Get_VolaVAPCODE: String;
begin
  Result:= sVolaVAPCODE;
end;

procedure TVolaSMS.Set_VolaVAPCODE(const Value: String);
begin
  sVolaVAPCODE:= Value;
end;

{*
 Invia un sms al gateway
 Il parametro DestPhone puo' contenere piu' destinatari che
 devono essere separati dal carattere "," (virgola)
*}
function TVolaSMS.SendSms(Sms: TSms): TSmsResponse;
var
 CmdResponse: TSmsResponse;
 Params: TIdMultiPartFormDataStream;
 Response: String;
 Date: String;
 Time: String;
begin
  CmdResponse:= TSmsResponse.create;
  if Sms<>nil then
  begin
   if Sms.DateTime>0 then
    begin
     DateTimeToString(Date, 'yyyy-mm-dd', Sms.DateTime);
     DateTimeToString(Time, 'hh:nn', Sms.DateTime);
     Time:=StringReplace(Time,'.',':',[rfReplaceAll]);
    end
   else
    begin
      Date:='0000-00-00';
      Time:='00:00';
    end;
   Params := TIdMultiPartFormDataStream.Create;
   Params.AddFormField('CMD', '1');
   Params.AddFormField('UID', VolaUser);
   Params.AddFormField('PWD', VolaPwd);
   Params.AddFormField('VCODE', VolaVAPCODE);
   Params.AddFormField('MITT', Sms.SenderPhone);
   Params.AddFormField('DEST', Sms.DestPhone);
   Params.AddFormField('DATE', Date);
   Params.AddFormField('TIME', Time);
   Params.AddFormField('MSG', Sms.Msg);
   if (VolaTestMode) then  Params.AddFormField('TEST', '1')
   else Params.AddFormField('TEST', '0');

   Request.ContentType := 'application/x-www-form-urlencoded';
   CmdResponse.Code := '00';
   try
     Response:= Post('http://' + Host + '/cgi/volasms_gw.php',Params);
     CmdResponse:=ParseResult(Response);
   except
     on E: Exception do CmdResponse.ErrMsg:='Error: ' + E.Message;
   end;
  end;
  Result := CmdResponse;

end;

function TVolaSMS.Get_VolaTestMode: boolean;
begin
  Result:=sVolaTestMode;
end;

procedure TVolaSMS.Set_VolaTestMode(const Value: boolean);
begin
  sVolaTestMode:=Value;
end;

{ *
 Restituisce i messaggi in entrata.
*}
function TVolaSMS.GetIncomingSms: TSmsResponse;
var
 CmdResponse: TSmsResponse;
 Params: TIdMultiPartFormDataStream;
begin
  CmdResponse:=TSmsResponse.create;

  Params := TIdMultiPartFormDataStream.Create;
  Params.AddFormField('CMD', '4');
  Params.AddFormField('UID', VolaUser);
  Params.AddFormField('PWD', VolaPwd);
  Params.AddFormField('VCODE', VolaVAPCODE);
  if (VolaTestMode) then  Params.AddFormField('TEST', '1')
  else Params.AddFormField('TEST', '0');
  Request.ContentType := 'application/x-www-form-urlencoded';
  CmdResponse.Code:='00';
  try
     CmdResponse:= ParseResult(Post('http://' + Host + '/cgi/volasms_gw.php',Params));
  except
    on E: Exception do CmdResponse.ErrMsg:='Error: ' + E.Message;
  end;

  Params.Free;
  Result := CmdResponse;
end;

{ *
 Restituisce il credito residuo di volasms.
*}
function TVolaSMS.GetCredit: integer;
var
 CmdResponse: TSmsResponse;
 Params: TIdMultiPartFormDataStream;

begin
  CmdResponse:=TSmsResponse.create;
  CmdResponse.Code:='00';
  CmdResponse:=SetAppVersion;

  Params := TIdMultiPartFormDataStream.Create;
  Params.AddFormField('CMD', '2');
  Params.AddFormField('UID', VolaUser);
  Params.AddFormField('PWD', VolaPwd);
  Params.AddFormField('VCODE', VolaVAPCODE);
  Request.ContentType := 'application/x-www-form-urlencoded';
  CmdResponse.Code:='00';
  try
     CmdResponse:= ParseResult(Post('http://' + Host + '/cgi/volasms_gw.php',Params));
  except
    on E: Exception do CmdResponse.ErrMsg:='Error: ' + E.Message;
  end;

  Params.Free;

  if CmdResponse.Code='01' then Result:=CmdResponse.SentNumErr
  else Result:=-1;
end;

{ *
 Aggiorna la versione lato server.
*}
function TVolaSMS.SetAppVersion: TSmsResponse;
var
 CmdResponse: TSmsResponse;
 Params: TIdMultiPartFormDataStream;

begin
  CmdResponse:=TSmsResponse.create;

  Params := TIdMultiPartFormDataStream.Create;
  Params.AddFormField('CMD', '3');
  Params.AddFormField('UID', VolaUser);
  Params.AddFormField('PWD', VolaPwd);
  Params.AddFormField('APP', VolaAppVer);
  Request.ContentType := 'application/x-www-form-urlencoded';
  CmdResponse.Code:='00';
  try
     CmdResponse:= ParseResult(Post('http://' + Host + '/cgi/volasms_gw.php',Params));
  except
    on E: Exception do CmdResponse.ErrMsg:='Error: ' + E.Message;
  end;

  Params.Free;
  Result:=CmdResponse;

end;

{*
 Effettua il parsing dei dati restituiti dal server
*}
function TVolaSMS.ParseResult(Value: String): TSmsResponse;
Var
  SmsIn: TSmsList;
  CmdResponse, CmdResponseAck: TSmsResponse;
  Params: TStringList;
  CmdParams: TStringList;
  numSmsIn, i: integer;
  SmsParse: TStringList;
  Sms: TSms;
begin
     CmdResponse:=TSmsResponse.create;

     CmdResponse.Code:='00';
     if ( (Pos('<BODY>',Value)>0) and (Pos('</BODY>',Value)>0) ) then
     begin
        value:=Copy(Value,Pos('<BODY>',Value)+8,Pos('</BODY>',Value)-2-Pos('<BODY>',Value)-8);
        params:=Explode(value, #13#10);
        if params.Count=1 then
        begin
          CmdParams:= Explode(params[0], ' ');
          if CmdParams.count=1 then
          begin
            CmdResponse.Code:=CmdParams[0];
            CmdResponse.SentNumErr:=0;
            CmdResponse.SentListErr:='';
            CmdResponse.ErrMsg:='';
          end;
          if CmdParams.count=2 then
          begin
            CmdResponse.Code:=CmdParams[0];
            CmdResponse.SentNumErr:=StrToInt(CmdParams[1]);
            CmdResponse.SentListErr:='';
            CmdResponse.ErrMsg:='';
          end;
          if CmdParams.count=3 then
          begin
            CmdResponse.Code:=CmdParams[0];
            CmdResponse.SentNumErr:=StrToInt(CmdParams[1]);
            CmdResponse.SentListErr:=CmdParams[2];
            CmdResponse.ErrMsg:='';
          end;
        end;
        if params.Count>1 then
        begin
          CmdParams:= Explode(params[0], ' ');
          // Risultato CMD
          CmdResponse.Code:=CmdParams[0];
          // Numero SmsIn
          SetLength(SmsIn,StrToInt(CmdParams[1]));
          // Elaboro la lista dei messaggi in arrivo se ve ne sono
          numSmsIn:= StrToInt(CmdParams[1]);
          if (numSmsIn>0) then
          begin
             for i:=1 to numSmsIn do
             begin
              SmsParse:= Explode(Params[i],#9);
              if SmsParse.Count = 4 then
              begin
                Sms:= TSms.Create;
                Sms.DateTime:=StrToDateTime(StringReplace(SmsParse[0],'-','/',[rfReplaceAll]) + ' ' + SmsParse[1]);
                Sms.SenderPhone:=SmsParse[2];
                Sms.DestPhone:='';
                Sms.Msg:=SmsParse[3];
                SmsIn[i-1]:=Sms;
              end;
             end;
          end;

          CmdResponse.SentNumErr:=0;
          CmdResponse.SentListErr:='';
          CmdResponse.SmsIncoming:=SmsIn;
          CmdResponse.ErrMsg:='';
          // Invio l'Ack della ricezione al server
          CmdResponseAck:=AckIncomingSms;
          if CmdResponseAck.Code<>'01' then CmdResponse.ErrMsg:='Warning: Ack to server error.';

        end;
        Params.Free;
     end;

     Result:= CmdResponse;
end;


function TVolaSMS.Explode(ch:string; sep:string = ';'):TStringList;
var 
p : integer; 
begin
p := pos(sep,ch); 
explode := TStringList.Create; 
while p > 0 do begin 
    explode.Add(copy(ch,1,p-1)); 
    if p <= length(ch) then ch := copy(ch,p+ length(sep),length(ch)); 
    p := pos(sep,ch); 
end; 
if length(ch)>0 then explode.Add(ch); 
end; 

function TVolaSMS.Implode(lst:TStringList; sep:string =';'):string;
var 
    i : integer; 
    s : string; 
begin
i:= 0; 
while i < lst.Count - 1 do begin 
     s := s + lst[i] + sep; 
     i := i + 1; 
end; 
if i < lst.Count then s := s + lst[i]; 
result := s; 
end;

function TVolaSMS.Get_VolaAppVer: String;
begin
   Result:='Delphi 1.0.0';
end;

function TVolaSMS.AckIncomingSms: TSmsResponse;
var
 CmdResponse: TSmsResponse;
 Params: TIdMultiPartFormDataStream;

begin
  CmdResponse:=TSmsResponse.create;

  Params := TIdMultiPartFormDataStream.Create;
  Params.AddFormField('CMD', '5');
  Params.AddFormField('UID', VolaUser);
  Params.AddFormField('PWD', VolaPwd);
  Params.AddFormField('VCODE', VolaVAPCODE);
  Request.ContentType := 'application/x-www-form-urlencoded';
  CmdResponse.Code:='00';
  try
     CmdResponse:= ParseResult(Post('http://' + Host + '/cgi/volasms_gw.php',Params));
  except
    on E: Exception do CmdResponse.ErrMsg:='Error: ' + E.Message;
  end;

  Params.Free;
  Result := CmdResponse;
end;

end.
